title: Principles and priorities
url: https://adactio.com/journal/16811
hash_url: f3be13f0057c9ff350b6e0bf3c3be90b

<p>I think about design principles a lot. I’m such a nerd for design principles, I even have <a href="https://principles.adactio.com">a collection</a>. I’m not saying all of the design principles in the collection are good—far from it! I collect them without judgement.</p>

<p>As for what makes a good design principle, <a href="https://adactio.com/journal/15559">I’ve written about that before</a>. One aspect that everyone seems to agree on is that a design principle shouldn’t be an obvious truism. Take this as an example:</p>

<blockquote>
  <p>Make it usable.</p>
</blockquote>

<p>Who’s going to disagree with that? It’s so agreeable that it’s practically worthless as a design principle. But now take this statement:</p>

<blockquote>
  <p>Usability is more important than profitability.</p>
</blockquote>

<p>Ooh, now we’re talking! That’s controversial. That’s bound to surface some disagreement, which is a good thing. It’s now passing the reversability test—it’s not hard to imagine an endeavour driven by the opposite:</p>

<blockquote>
  <p>Profitability is more important than usability.</p>
</blockquote>

<p>In either formulation, what makes these statements better than the bland toothless agreeable statements—“Usability is good!”, “Profitability is good!”—is that they introduce the element of prioritisation.</p>

<p>I like design principles that can be formulated as:</p>

<blockquote>
  <p>X, even over Y.</p>
</blockquote>

<p>It’s not saying that Y is unimportant, just that X is <em>more</em> important:</p>

<blockquote>
  <p>Usability, even over profitability.</p>
</blockquote>

<p>Or:</p>

<blockquote>
  <p>Profitability, even over usability.</p>
</blockquote>

<p>Design principles formulated this way help to crystalise priorities. <a href="https://clearleft.com/posts/getting-your-priorities-right">Chris has written about the importance of establishing—and revisiting—priorities on any project</a>:</p>

<blockquote>
  <p>Prioritisation isn’t and shouldn’t be a one-off exercise. The changing needs of your customers, the business environment and new opportunities from technology mean prioritisation is best done as a regular activity.</p>
</blockquote>

<p>I’ve said it many times, but one on my favourite design principles comes from the <a href="https://www.w3.org/TR/html-design-principles/">HTML design principles</a>. The <a href="https://www.w3.org/TR/html-design-principles/#priority-of-constituencies">priority of consitituencies</a> (it’s got “priorities” right there in the name!):</p>

<blockquote>
  <p>In case of conflict, consider users over authors over implementors over specifiers over theoretical purity.</p>
</blockquote>

<p>Or put another way:</p>

<ul>
<li>Users, even over authors.</li>
<li>Authors, even over implementors.</li>
<li>Implementors, even over specifiers.</li>
<li>Specifiers, even over theoretical purity.</li>
</ul>

<p>When it comes to <a href="https://adactio.com/articles/12839">evaluating technology</a> for the web, I think there are a number of factors at play.</p>

<p>First and foremost, there’s the end user. If a technology choice harms the end user, avoid it. I’m thinking here of the kind of performance tax that a user has to pay when developers choose to use megabytes of JavaScript.</p>

<p>Mind you, some technologies have no direct effect on the end user. When it comes to build tools, version control, toolchains …all the stuff that sits on your computer and never directly interacts with users. In that situation, the wants and needs of developers can absolutely take priority.</p>

<p>But as a general principle, I think this works:</p>

<blockquote>
  <p>User experience, even over developer experience.</p>
</blockquote>

<p>Sadly, I think the current state of “modern” web development reverses that principle. Developer efficiency is prized above all else. Like I said, that would be absolutely fine if we’re talking about technologies that only developers are exposed to, but as soon as we’re talking about shipping those technologies over the network to end users, it’s negligent to continue to prioritise the developer experience.</p>

<p>I feel like personal websites are an exception here. What you do on your own website is completely up to you. But once you’re taking a paycheck to make websites that will be used by other people, it’s incumbent on you to realise that it’s not about you.</p>

<p>I’ve been talking about developers here, but this is something that applies just as much to designers. But I feel like designers go through that priority shift fairly early in their career. At the outset, they’re eager to make their mark and prove themselves. As they grow and realise that it’s not about them, they understand that the most appropriate solution for the user is what matters, even if that’s a “boring” tried-and-tested pattern that isn’t going to wow any fellow designers.</p>

<p>I’d like to think that developers would follow a similar progression, and I’m sure that some do. But I’ve seen many senior developers who have grown <em>more</em> enamoured with technologies instead of honing in on the most appropriate technology for end users. Maybe that’s because in many organisations, developers are positioned further away from the end users (whereas designers are ideally being confronted with their creations being used by actual people). If a lead developer is focused on the productivity, efficiency, and happiness of the dev team, it’s no wonder that their priorities end up overtaking the user experience.</p>

<p>I realise I’m talking in very binary terms here: developer experience versus user experience. I know it’s not always that simple. Other priorities also come into play, like business needs. Sometimes business needs are in direct conflict with user needs. If an online business makes its money through invasive tracking and surveillance, then there’s no point in having a design principle that claims to prioritise user needs above all else. That would be a hollow claim, and the design principle would become worthless.</p>

<p>Because that’s the point with design principles. They’re there to be used. They’re not a nice fluffy exercise in feeling good about your work. The priority of constituencies begins, “in case of conflict” and that’s exactly when a design principle matters—when it’s tested.</p>

<p>Suppose someone with a lot of clout in your organisation makes a decision, but that decision conflicts with your organisations’s design principles. Instead of having an opinion-based argument about who’s right or wrong, the previously agreed-upon design principles allow you to take ego out of the equation.</p>

<p>Prioritisation isn’t easy, and it gets harder the more factors come into play: user needs, business needs, technical constraints. But it’s worth investing the time to get agreement on the priority of your constituencies. And then formulate that agreement into design principles.</p>
